Scriptname StashCapsMonitor extends ReferenceAlias

;===========================================================
; STASH THOSE CAPS
; by Greslin
; v2.3
;===========================================================

Actor PlayerRef

bool BarterMenuOn = false

int CapInterval = 65535		; count limit on additem().  anything above this rolls the count over.

MiscObject Property Caps001 Auto Const
MiscObject Property CapCurrency Auto Const

Event OnInit()

	if Game.IsPluginInstalled("Stashable Caps.esp") && \
		Game.IsPluginInstalled("Stashable Caps - Weighted.esp")
		debug.messagebox("WARNING: You have both the weighted and unweighted versions of Stash Those Caps loaded!  This will cause problems!  Quit now and uncheck one of the ESPs in your load order.")
	else
		Startup()
	endif
EndEvent

Event Actor.OnPlayerLoadGame(Actor akSender)
	if Game.IsPluginInstalled("Stashable Caps.esp") && \
		Game.IsPluginInstalled("Stashable Caps - Weighted.esp")
		debug.messagebox("WARNING: You have both the weighted and unweighted versions of Stash Those Caps loaded!  This will cause problems!  Quit now and uncheck one of the ESPs in your load order.")
	else
		Startup()
	endif
EndEvent

Function Startup()

	PlayerRef = Game.GetPlayer()
	int CCaps = PlayerRef.GetItemCount(Caps001)
	
	debug.OpenUserLog("StashCaps")
	debug.traceuser("StashCaps", "Stash Those Caps 2.3 (4/5/20) started.")
	debug.traceuser("StashCaps", "Player currently has " + CCaps + " caps in inventory.")
	;debug.traceuser("StashCaps", "Start state = " + GetState() + ".")
	
	; deal with whatever caps are already in possession
	RemoveAllItems(CapCurrency)
	AddLongItems(CapCurrency, CCaps)
	
	; tap events
	AddInventoryEventFilter(Caps001) 
	AddInventoryEventFilter(CapCurrency)
	RegisterForMenuOpenCloseEvent("BarterMenu")
endFunction

; note: Remove and SetFilter have to be set within these events to avoid recursion.  it was a nasty bug in v1.

State NoCapFilter
	Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
	EndEvent

	Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
	EndEvent
EndState

Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
	;debug.traceuser("StashCaps", "OnAdded entered, state = " + GetState() + ".")
	if !BarterMenuOn
		if RemoveFilters()
		
			;/
				if we added 100000 caps in one throw, we actually only get the remainder 34464 (100000 - 65536).
				we need to populate the rest by hand
			/;
			
			debug.traceuser("StashCaps", aiItemCount + " of " + akBaseItem + " added to inventory.")
					
			if aiItemCount > CapInterval
				int mod = aiItemCount % (CapInterval + 1)	; the remainder value that actually got added
				debug.traceuser("StashCaps", ".. unfortunately only " + mod + " actually got added.")
				AddLongItems(akBaseItem as MiscObject, aiItemCount - mod)	; this should always be an even multiple of 65536
			endif

			; now we should be good.  sync up.
			
			SyncCaps(akBaseItem)
			SetFilters()
		endif
	endif
EndEvent

Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
	;debug.traceuser("StashCaps", "OnRemove entered, state = " + GetState() + ".")
	
	if !BarterMenuOn
		if RemoveFilters()
			debug.traceuser("StashCaps", aiItemCount + " of " + akBaseItem + " removed from inventory.")
			
			; if akDestContainer != none
				if aiItemCount > CapInterval
					int mod = aiItemCount % (CapInterval + 1)	; the remainder value that actually got added
					debug.traceuser("StashCaps", ".. unfortunately only " + aiItemCount + " actually got removed.")
					AddLongItems(akBaseItem as MiscObject, aiItemCount - mod, akDestContainer)	; this should always be an even multiple of 65536
				endif
			; endif
			
			SyncCaps(akBaseItem)
			SetFilters()
		endif
	endif
EndEvent



; Prevent currency from making an appearance in trading screens

Event OnMenuOpenCloseEvent(string asMenuName, bool abOpening)
	Int CCaps

	BarterMenuOn = abOpening
	
	if abOpening
		if RemoveFilters()
			debug.traceuser("StashCaps", "Barter menu opened.")
			RemoveAllItems(CapCurrency)
		endif
	else
		debug.traceuser("StashCaps", "Barter menu closed.")
		SyncCurToCaps()
		SetFilters()	; set things back to normal
	endif
EndEvent

; NOTHING should be kicking off the Add/Remove events after this point.  Filters should be OFF.

Function SyncCaps(Form akBaseItem)
	debug.traceuser("StashCaps", "SyncCaps() called on " + akBaseItem)
	if akBaseItem == Caps001
		SyncCurToCaps()
	elseif akBaseItem == CapCurrency
		SyncCapsToCur()
	endif
endFunction

; Set Currency 

Function SyncCurToCaps()

	int CCaps

	CCaps = PlayerRef.GetItemCount(Caps001)
	RemoveAllItems(CapCurrency)
	AddLongItems(CapCurrency, CCaps)
	debug.traceuser("StashCaps", "SyncCurToCaps() called, player now has " + CCaps + " caps.")
EndFunction

; Set Caps

Function SyncCapsToCur()
	Int CCaps
	
	CCaps = PlayerRef.GetItemCount(CapCurrency)
	RemoveAllItems(Caps001)
	AddLongItems(Caps001, CCaps)	
	debug.traceuser("StashCaps", "SyncCapsToCur() called, player now has " + CCaps + " caps.")
EndFunction

; note: using additem to add > 65535 results in only getting the remainder.
; for example, additem 100000 results in 34464
; now the weird thing is that this doesn't seem to be true for RemoveItem
; so anytime we're doing an add, we have to break it down into usable subtransactions

Function AddLongItems(MiscObject Cur, Int CCaps, ObjectReference ThisContainer = none)
	
	if !ThisContainer
		ThisContainer = PlayerRef
	endif
	
	if CCaps 
		debug.traceuser("StashCaps", "AddLongItems() called, transferring " + CCaps + " caps.")
		
		while CCaps > CapInterval
			CCaps -= CapInterval
			ThisContainer.additem(Cur, CapInterval, true)
			debug.traceuser("StashCaps", "AddLongItems(): added interval " + CapInterval + " of " + CCaps + " " + Cur)
		endwhile
		
		debug.traceuser("StashCaps", "AddLongItems(): adding remainder " + CCaps + " " + Cur)
		ThisContainer.additem(Cur, CCaps, true)
	else
		debug.traceuser("StashCaps", "AddLongItems() called, aborted for zero caps.")
	endif
endFunction

Function RemoveAllItems(MiscObject Cur)
	PlayerRef.removeitem(Cur, 999999999, true)
	debug.traceuser("StashCaps", "RemoveAllItems() called on " + Cur)
endFunction

Function SetFilters()
	if GetState() == "NoCapFilter"
		RemoveInventoryEventFilter(Caps001) 
		RemoveInventoryEventFilter(CapCurrency)
		GoToState("")
		AddInventoryEventFilter(Caps001) 
		AddInventoryEventFilter(CapCurrency)
		debug.traceuser("StashCaps", "Inventory filters set.")
	endif
endFunction

bool Function RemoveFilters()
	if GetState() == "" 
		RemoveInventoryEventFilter(Caps001) 
		RemoveInventoryEventFilter(CapCurrency)
		GotoState("NoCapFilter")
		debug.traceuser("StashCaps", "Inventory filters removed.")
		return true
	endif
	return false
endFunction

